"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const winston_1 = require("winston");
const app_root_path_1 = __importDefault(require("app-root-path"));
const { combine, timestamp, label, printf } = winston_1.format;
const tagLen = 8;
const faceLen = 6;
const levelSign = {
    info: 'ℹ️ ',
    warn: '⚠️ ',
    error: '⛔️',
    debug: '☢️ '
};
class ServiceLogger {
    constructor(serviceName, instanceName, path, face, config) {
        this.serviceName = '';
        this.instanceName = '';
        this.path = 'logs';
        this.face = '';
        this.logger = null;
        this.config = {
            showService: true,
            silent: false
        };
        this.info = (message) => {
            if (this.logger != null && !this.config.silent)
                this.logger.info(message);
        };
        this.warn = (message) => {
            if (this.logger != null && !this.config.silent)
                this.logger.warn(message);
        };
        this.err = (message) => {
            if (this.logger != null && !this.config.silent)
                this.logger.error(message);
        };
        this.debug = (message) => {
            if (this.logger != null)
                this.logger.debug(message);
        };
        this.silent = (mod) => {
            this.config.silent = mod;
        };
        if (config) {
            this.config = Object.assign({}, this.config, config);
        }
        if (serviceName.length <= tagLen) {
            this.serviceName = serviceName;
        }
        else {
            console.error(`Service name cannot be longer than ${tagLen} characters`);
            process.exit(1);
        }
        if (instanceName.length <= tagLen) {
            if (instanceName.length === 0) {
                this.instanceName = '';
            }
            else {
                this.instanceName = instanceName;
            }
        }
        else {
            console.error(`Instance name cannot be longer than ${tagLen} characters`);
            process.exit(1);
        }
        if (!face) {
        }
        else if (face.length > faceLen) {
            this.face = face;
        }
        else {
            console.error(`Face string cannot be longer than ${tagLen} characters`);
            process.exit(1);
        }
        this.path = path || this.path;
        const style = printf((info) => {
            const level = levelSign[info.level];
            const sern = this.serviceName !== ''
                ? '@' + this.serviceName.toUpperCase() + Array(tagLen + 1 - this.serviceName.length).join(' ')
                : Array(tagLen + 2).join(' ');
            const insn = this.instanceName !== ''
                ? '#' + this.instanceName.toUpperCase() + Array(tagLen + 1 - this.instanceName.length).join(' ')
                : Array(tagLen + 2).join(' ');
            const now = new Date();
            return (`${this.config.showService ? sern : ''} ${insn}` +
                `[${(now.getHours() < 10 ? ' ' : '') + now.getHours()}:${(now.getMinutes() < 10 ? ' ' : '') +
                    now.getMinutes()}]` +
                `${this.face} ${level}| ${info.message}`);
        });
        const options = {
            file: {
                filename: `${app_root_path_1.default}/${this.path}/${this.serviceName}.log`,
                handleExceptions: true,
                json: true,
                maxsize: 5242880,
                maxFiles: 1,
                colorize: false
            },
            console: {
                handleExceptions: true,
                json: false,
                colorize: true
            }
        };
        this.logger = winston_1.createLogger({
            format: combine(timestamp(), style),
            transports: [new winston_1.transports.File(options.file), new winston_1.transports.Console(options.console)],
            exitOnError: false
        });
    }
}
class LogManager {
    constructor(serviceName, path, config) {
        this.config = {
            showService: true
        };
        this.instances = [];
        this.getLogger = (instanceName, config) => {
            const instance = new ServiceLogger(this.serviceName, instanceName || '', this.path, undefined, Object.assign({}, config, this.config));
            this.instances.push(instance);
            return instance;
        };
        this.mute = () => {
            this.instances.forEach((logger) => {
                logger.silent(true);
            });
        };
        this.unmute = () => {
            this.instances.forEach((logger) => {
                logger.silent(false);
            });
        };
        if (config) {
            this.config = Object.assign({}, this.config, config);
        }
        this.serviceName = serviceName;
        this.path = path || 'logs';
    }
}
exports.default = LogManager;
//# sourceMappingURL=index.js.map